# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cryptography import fernet
import msgpack
import six


def _restore_padding(token):
    """Restore padding based on token size.

    :param token: token to restore padding on
    :returns: token with correct padding

    """
    # Re-inflate the padding
    mod_returned = len(token) % 4
    if mod_returned:
        missing_padding = 4 - mod_returned
        token += b'=' * missing_padding
    return token


def unpack_token(fernet_token, fernet_keys):
    """Attempt to unpack a token using the supplied Fernet keys.

    :param fernet_token: token to unpack
    :type fernet_token: string
    :param fernet_token: a list consisting of keys in the repository
    :type fernet_token: string
    :returns: the token payload
    :raises: Exception in the event the token can't be unpacked

    """
    # create a list of fernet instances
    fernet_instances = [fernet.Fernet(key) for key in fernet_keys]
    # create a encryption/decryption object from the fernet keys
    crypt = fernet.MultiFernet(fernet_instances)

    # attempt to decode the token
    token = _restore_padding(six.binary_type(fernet_token))
    serialized_payload = crypt.decrypt(token)
    payload = msgpack.unpackb(serialized_payload)

    translated_payload = []
    for item in payload:
        if isinstance(item, list):
            translated_item = []
            for i in item:
                translated_item.append(i)
            translated_payload.append(translated_item)
        else:
            translated_payload.append(item)

    # present token values
    return translated_payload



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import os

import fernet_inspector


def _get_args():
    """Get arguments from the user."""
    parser = argparse.ArgumentParser(
        prog='fernet-inspector',
        description='Inspect the contents of a Keystone Fernet token from '
                    'the host it was issued from.')
    parser.add_argument('token', type=str, help='token to decrypt')
    parser.add_argument('-k', '--key-repository', type=str,
                        default='/etc/keystone/fernet-keys/',
                        help='location of Fernet key repository.')
    args = parser.parse_args()
    return args.token, args.key_repository


def _read_keys_from_disk(key_repository):
    """Read the keys from the key repository."""
    # validate the repository location
    if not os.path.isdir(key_repository):
        raise fernet_inspector.NonExistentKeyRepository()

    # build a dictionary of key_number:encryption_key pairs
    keys = dict()
    for filename in os.listdir(key_repository):
        path = os.path.join(key_repository, str(filename))
        if os.path.isfile(path):
            with open(path, 'r') as key_file:
                try:
                    key_id = int(filename)
                except ValueError:
                    pass
                else:
                    keys[key_id] = key_file.read()

    # return a list of key values
    return [keys[x] for x in sorted(keys.keys(), reverse=True)]


def main():
    """The main entry point into the fernet_inspector."""
    # get arguments
    fernet_token, key_repository = _get_args()

    # build key set
    fernet_keys = _read_keys_from_disk(key_repository)

    # pass required stuff to the inspector and print the result
    print fernet_inspector.unpack_token(fernet_token, fernet_keys)



import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


HERE = os.path.dirname(__file__)


def read_requirements(filename):
    filename = os.path.join(HERE, filename)
    return [line.strip() for line in open(filename) if line.strip()]


setup(
    name='fernet_inspector',
    version='0.1.3',
    description='inspect fernet tokens generated by keystone',
    packages=['fernet_inspector'],
    entry_points={
        'console_scripts': [
            'fernet-inspector = fernet_inspector.cli:main'
        ]
    },
    long_description=open('README.rst').read(),
    author='Lance Bragstad',
    author_email='lbragstad@gmail.com',
    url='https://github.com/lbragstad/fernet-inspector',
    license='Apache Software License',
    install_requires=read_requirements('requirements.txt'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Testing',
    ]
)
